#include "hashedip"

unsigned HashedIp::target(struct in_addr clientip,
		     BackendVector const &targetlist) {
    // Hash the client's IP into an index
    unsigned h = 0;
    for (char *cp = (char*)&clientip;
	 unsigned(cp - (char*)&clientip) < sizeof(struct in_addr);
	 cp++) {
	h += *cp;
	h %= targetlist.size();
    }
    unsigned index = targetlist[h];

    msg ((Mstr("Client IP ") + static_cast<string>(inet_ntoa(clientip))) +
	 (Mstr(" hashes to ") + index) +
	 ", back end " + balancer.backend(index).description() +
	 "\n");

    // In strict mode, back end must be available, or don't proceed
    // In lax mode, fall back to least-connections dispatching
    if (! balancer.backend(index).available()) {
	if (config.dispatchmode() == Dispatchmode::m_strict_hashed_ip)
	    throw static_cast<Error>("Hashed-IP algorithm: target back end ") +
		   balancer.backend(index).description() + " unavailable";
	else {
	    msg ("Hashed-IP algorithm: target back end " +
		 balancer.backend(index).description() + " unavailable, "
		 "falling back to least-connections\n");
	    Leastconn l;
	    index = l.target(clientip, targetlist);
	}
    }

    // Got it
    return (index);
}
