#include "backend"

bool Backend::connect() {
    PROFILE("Backend::connect");
    
    // Assume the backend is dead
    islive = false;

    debugmsg ("Connecting to back end " + description() + "\n");
    
    // Create client socket
    if ( (clsocket = socket (PF_INET, SOCK_STREAM, 0)) < 0 )
	throw static_cast<Error>("Failed to create client socket: ") +
	    strerror(errno);

    // Resolve hostname, prepare binding
    struct sockaddr_in backendaddr;
    
    backendaddr.sin_family = AF_INET;
    backendaddr.sin_port = htons(bdef.port());
    backendaddr.sin_addr.s_addr = dnsentry.resolve(bdef.server());

    // Client socket goes into nonblocking mode, so we can connect
    // and enforce a timeout later.
    int flags;
    if ( (flags = fcntl (clsocket, F_GETFL, 0)) == -1 ) {
	socketclose (clsocket);
	throw static_cast<Error>("Failed to get fd flags: ") + strerror(errno);
    }    
    if (fcntl (clsocket, F_SETFL, flags | O_NONBLOCK) == -1) {
	socketclose (clsocket);
	throw static_cast<Error>("Failed to fd in nonblocking mode: ") +
	    strerror(errno);
    }

    // Do the connect
    int conres = ::connect (clsocket, (struct sockaddr *)&backendaddr,
			 sizeof(backendaddr));
    int conerrno = errno;
    debugmsg((Mstr("Connect result: ") + conres) +
	     (Mstr(", errno: ") + conerrno) + "\n");

    // Put socket again in blocking mode.
    if (fcntl (clsocket, F_SETFL, flags) == -1) {
	socketclose (clsocket);
	throw static_cast<Error>("Failed to put fd in blocking mode: ") +
	    strerror(errno);
    }
    
    // Check on the outcome of the connect
    if (!conres || conerrno == EINPROGRESS) {
	// Wait for socket to go writable.
	Fdset fdset (config.backend_timeout());
	fdset.add (clsocket);
	int rwsock = fdset.readwriteable();
	int wsock  = fdset.writeable();
	debugmsg (Mstr("Connecting to ") + description() +
		  Mstr(Mstr(": writesocket=") + wsock) +
		  Mstr(Mstr(", read/writesocket=") + rwsock) + "\n");
	if (wsock == clsocket && rwsock == -1)
	    islive = true;
    }

    debugmsg ((Mstr("Back end ") + description()) +
	      (Mstr(" is ") + livestr()) +
	      (Mstr(" (socket ") + clsocket) + ")\n");
    
    return (islive);
}
