#include "fdset"

int Fdset::readable() const {
    PROFILE("Fdset::readable");
    
    fd_set readset, exceptset;
    struct timeval tv, *tvp;

    // No fd's? Nothing is readable.
    if (set.size() < 1)
	return (-1);

    if (config.debug()) {
	ostringstream o;
	o << "Candidate readable fd's:";
	for (unsigned i = 0; i < set.size(); i++)
	    o << ' ' << set[i];
	_debugmsg(o.str() + '\n');
    }

    // Prepare select sets.
    FD_ZERO (&readset);
    FD_ZERO (&exceptset);
    for (unsigned i = 0; i < set.size(); i++) {
	FD_SET (set[i], &readset);
	FD_SET (set[i], &exceptset);
    }

    // Prepare timout specifier.
    if (tsec) {
	tv.tv_sec = tsec;
	tv.tv_usec = 0;
	tvp = &tv;
    } else
	tvp = 0;

    // Run the select. Signal interrupts are returned as -1, so that
    // the caller can handle them gracefully.
    if (select (FD_SETSIZE, &readset, 0, &exceptset, tvp) < 0) {
	debugmsg (Mstr("Select interrupted with errno ") + errno +
		  " while waiting for readable fd\n");
	if (errno != EINTR)
	    throw static_cast<Error>
		("Select failure: failed to wait for readable state: ") +
		strerror(errno);
	else
	    return (-1);
    }

    // Check for exceptions.
    for (unsigned i = 0; i < set.size(); i++)
	if (FD_ISSET (set[i], &exceptset))
	    throw static_cast<Error>("Exception on fd/socket ") + int(set[i]);

    // Check what's readable.
    for (unsigned i = 0; i < set.size(); i++)
	if (FD_ISSET (set[i], &readset)) {
	    debugmsg (Mstr("Fd ") + set[i] + " has become readable\n");
	    return (set[i]);
	}
    
    // Nothing..
    debugmsg ("No readable fd's at this time\n");
    return (-1);
}
    
