#include "netbuffer"
#include "balancer/balancer"

unsigned Netbuffer::netwrite (int fd, int timeout) const {
    PROFILE("Netbuffer::netwrite");

    debugmsg((Mstr("About to write ") + buf_sz) +
	     (Mstr(" bytes to fd ") + fd) +
	     (Mstr(", timeout ") + timeout) + "\n");
    
    if (!buf_sz)
	return (0);

    // Log to dump directory if requested
    if (config.dumpdir().length()) {
	ostringstream of;
	of << config.dumpdir() << "/" << balancer.requestnr() << "." << fd;
	FILE *f;
	if ( (!(f = fopen (of.str().c_str(), "a"))) &&
	     (!(f = fopen (of.str().c_str(), "w"))) )
	    warnmsg ("Cannot write traffic log " + of.str() + ": " +
		     strerror(errno) + "\n");
	else {
	    fwrite (buf_data, 1, buf_sz, f);
	    fclose (f);
	}
    }

    // Send to the socket
    unsigned totwritten = 0;
    while (totwritten < buf_sz) {
	// Wait for the socket to become writeable.
	if (timeout) {
	    Fdset set (timeout);
	    set.add (fd);
	    if (set.writeable() != fd)
		throw static_cast<Error>("Fd ") + fd +
		    " failed to become writable within " + timeout + " sec";
	}

	// Push bytes
	ssize_t nwritten;
	nwritten = write (fd, buf_data + totwritten, buf_sz - totwritten);

	if (config.debug() && nwritten > 0) {
	    ostringstream o;
	    o << "Sent " << nwritten << " bytes to fd " << fd << ": ";
	    for (unsigned i = totwritten; i < totwritten + nwritten; i++)
		o << printable(buf_data[i]);
	    o << "\n";
	    _debugmsg (o.str());
	}

	// EINVAL / EINPROGRESS errors are handled as: retry
	// If any bytes were written, we're ok
	if (nwritten >= 1)
	    totwritten += nwritten;
	else if (errno != EINVAL && errno != EINPROGRESS)
	    throw static_cast<Error>("Write/send failed: errno=") +
		int(errno) + ", " +  strerror(errno) +
		", result=" + (int)nwritten;
    }

    return buf_sz;
}

